'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var name = "@jitsu/pipeline-helpers";
var version = "0.4.1-alpha.2+4a5222c";
var description = "Utility functions designed to outsource some processing logic from Server to JS";
var author = "Jitsu Dev Team <dev@jitsu.com>";
var publishConfig = {
	access: "public"
};
var homepage = "https://github.com/jitsucom/jitsu-sdk#readme";
var license = "MIT";
var main = "lib/index.js";
var module$1 = "lib/index.es.js";
var types = "lib/index.d.ts";
var files = [
	"lib/*"
];
var repository = {
	type: "git",
	url: "https://github.com/jitsucom/jitsu-sdk",
	directory: "packages/pipeline-helpers"
};
var scripts = {
	build: "rm -rf ./lib && rollup -c && mv ./lib/src/*.d.ts ./lib && rmdir ./lib/src",
	test: "jest"
};
var bugs = {
	url: "https://github.com/jitsucom/jitsu/issues"
};
var devDependencies = {
	"@rollup/plugin-json": "^4.1.0",
	"@segment/analytics-next": "^1.32.1",
	"@types/jquery": "^3.5.10"
};
var gitHead = "4a5222ce49da3cb6be58648c78b1111b39764fe1";
var pkg = {
	name: name,
	version: version,
	description: description,
	author: author,
	publishConfig: publishConfig,
	homepage: homepage,
	license: license,
	main: main,
	module: module$1,
	types: types,
	files: files,
	repository: repository,
	scripts: scripts,
	bugs: bugs,
	devDependencies: devDependencies,
	gitHead: gitHead
};

var segmentEventsTypes = {
    alias: true,
    group: true,
    identify: true,
    page: true,
    screen: true,
    track: true
};
var getSegmentType = function (jitsuType, customMapping, defaultSegmentType) {
    if (jitsuType === "user_identity" || jitsuType === "identify") {
        return "identify";
    }
    else if (jitsuType === "page" || jitsuType === "pageview") {
        return "page";
    }
    else if (segmentEventsTypes[jitsuType] !== undefined) {
        return jitsuType;
    }
    else {
        var customType = customMapping[jitsuType];
        return customType !== null && customType !== void 0 ? customType : defaultSegmentType;
    }
};
function removeProps(ev, props) {
    var copy = __assign({}, ev);
    props.forEach(function (p) { return delete copy[p]; });
    return copy;
}
var jitsuToSegment = function (ev, _a) {
    var _b, _c, _d, _e, _f, _g, _h, _j;
    var _k = _a === void 0 ? {} : _a, _l = _k.defaultSegmentType, defaultSegmentType = _l === void 0 ? "track" : _l, _m = _k.eventTypeMapping, eventTypeMapping = _m === void 0 ? {} : _m;
    if (ev.src_payload && ev.src === "ajs") {
        return ev.src_payload;
    }
    var segmentEvent = {
        type: getSegmentType(ev.event_type, eventTypeMapping, defaultSegmentType),
        _metadata: {},
        anonymousId: (_b = ev.user) === null || _b === void 0 ? void 0 : _b.anonymous_id,
        context: {
            ip: ev.source_ip,
            locale: ev.user_language,
            userAgent: ev.user_agent,
            page: {
                path: ev.doc_path,
                referrer: ev.referer,
                search: ev.doc_search,
                title: ev.page_title,
                url: ev.url
            },
            campaign: {
                name: (_c = ev.utm) === null || _c === void 0 ? void 0 : _c.name,
                term: (_d = ev.utm) === null || _d === void 0 ? void 0 : _d.term,
                source: (_e = ev.utm) === null || _e === void 0 ? void 0 : _e.source,
                medium: (_f = ev.utm) === null || _f === void 0 ? void 0 : _f.medium,
                content: (_g = ev.utm) === null || _g === void 0 ? void 0 : _g.content
            }
        },
        event: ev.event_type,
        groupId: undefined,
        integrations: undefined,
        messageId: ev.event_id,
        name: "",
        properties: flatten(removeProps(ev, [
            "user",
            "source_ip",
            "user_language",
            "user_agent",
            "doc_path",
            "referer",
            "doc_search",
            "page_title",
            "url",
            "event_type",
            "event_id",
            "utc_time",
            "_timestamp",
            "eventn_ctx_event_id",
            "utm",
        ])),
        sentAt: ev.utc_time ? new Date(ev.utc_time) : new Date(),
        timestamp: ev.utc_time ? new Date(ev.utc_time) : new Date(),
        traits: flatten(removeProps(ev.user || {}, ["anonymous_id", "id"])),
        userId: ((_h = ev.user) === null || _h === void 0 ? void 0 : _h.id) || ((_j = ev.user) === null || _j === void 0 ? void 0 : _j.email)
    };
    removeEmptyFields(segmentEvent);
    return segmentEvent;
};
function safeToString(obj) {
    if (obj === undefined || obj === null) {
        return undefined;
    }
    else if (typeof obj === "string") {
        return obj;
    }
    else if (typeof obj === "object") {
        return JSON.stringify(obj);
    }
    else {
        return obj + "";
    }
}
var segmentTables = {
    track: "tracks",
    page: "pages",
    identify: "identifies",
    group: "groups",
    alias: "aliases",
    screen: "screens"
};
var segmentToTable = function (event, jitsuEvent) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10;
    var payloadObj = ((_a = jitsuEvent === null || jitsuEvent === void 0 ? void 0 : jitsuEvent.src_payload) === null || _a === void 0 ? void 0 : _a.obj) || {};
    return __assign({ JITSU_TABLE_NAME: segmentTables[event.type] || "tracks", __sql_type_timestamp: "timestamp", __sql_type_sent_at: "timestamp", anonymous_id: safeToString(event.anonymousId) || ((_b = jitsuEvent === null || jitsuEvent === void 0 ? void 0 : jitsuEvent.src_payload) === null || _b === void 0 ? void 0 : _b.anonymousId), context_campaign_source: (_d = (_c = event.context) === null || _c === void 0 ? void 0 : _c.campaign) === null || _d === void 0 ? void 0 : _d.source, context_ip: (_e = event.context) === null || _e === void 0 ? void 0 : _e.ip, context_library_name: ((_g = (_f = jitsuEvent === null || jitsuEvent === void 0 ? void 0 : jitsuEvent.context) === null || _f === void 0 ? void 0 : _f.library) === null || _g === void 0 ? void 0 : _g.version) || "jitsu-bridge", context_library_version: pkg.version , context_locale: ((_h = event.context) === null || _h === void 0 ? void 0 : _h.locale) || ((_j = payloadObj.context) === null || _j === void 0 ? void 0 : _j.locale), context_page_path: (_l = (_k = event.context) === null || _k === void 0 ? void 0 : _k.page) === null || _l === void 0 ? void 0 : _l.path, context_page_referrer: (_o = (_m = event.context) === null || _m === void 0 ? void 0 : _m.page) === null || _o === void 0 ? void 0 : _o.referrer, context_page_search: (_q = (_p = event.context) === null || _p === void 0 ? void 0 : _p.page) === null || _q === void 0 ? void 0 : _q.search, context_page_title: (_s = (_r = event.context) === null || _r === void 0 ? void 0 : _r.page) === null || _s === void 0 ? void 0 : _s.title, context_page_url: (_u = (_t = event.context) === null || _t === void 0 ? void 0 : _t.page) === null || _u === void 0 ? void 0 : _u.url, context_user_agent: (_v = event.context) === null || _v === void 0 ? void 0 : _v.userAgent, context_utm_source: (_x = (_w = event.context) === null || _w === void 0 ? void 0 : _w.campaign) === null || _x === void 0 ? void 0 : _x.source, email: safeToString((_y = event.traits) === null || _y === void 0 ? void 0 : _y.email), id: event.messageId, name: safeToString((_z = event.traits) === null || _z === void 0 ? void 0 : _z.name), path: (_1 = (_0 = event.context) === null || _0 === void 0 ? void 0 : _0.page) === null || _1 === void 0 ? void 0 : _1.path, referrer: (_3 = (_2 = event.context) === null || _2 === void 0 ? void 0 : _2.page) === null || _3 === void 0 ? void 0 : _3.referrer, search: (_5 = (_4 = event.context) === null || _4 === void 0 ? void 0 : _4.page) === null || _5 === void 0 ? void 0 : _5.search, sent_at: safeToString(event.sentAt), timestamp: payloadObj.timestamp || safeToString(event.timestamp), title: (_7 = (_6 = event.context) === null || _6 === void 0 ? void 0 : _6.page) === null || _7 === void 0 ? void 0 : _7.title, url: (_9 = (_8 = event.context) === null || _8 === void 0 ? void 0 : _8.page) === null || _9 === void 0 ? void 0 : _9.url, user_id: safeToString(event.userId) || ((_10 = jitsuEvent === null || jitsuEvent === void 0 ? void 0 : jitsuEvent.src_payload) === null || _10 === void 0 ? void 0 : _10.userId) }, (event.properties || {}));
};
function canonicalSqlTypeHint(hint) {
    if (typeof hint === "object") {
        if (!Array.isArray(hint)) {
            return hint;
        }
        else {
            return { defaultCastType: "hint[0]", defaultDDLType: "hint[1]", dialects: {} };
        }
    }
    else {
        return { defaultCastType: hint, defaultDDLType: hint, dialects: {} };
    }
}
function removeEmptyFields(obj) {
    for (var _i = 0, _a = Object.entries(obj); _i < _a.length; _i++) {
        var _b = _a[_i], key = _b[0], val = _b[1];
        if (val !== null && typeof val === "object" && !Array.isArray(val)) {
            removeEmptyFields(val);
        }
        else if (val === undefined) {
            delete obj[key];
        }
    }
}
function flatten(obj, _a, path) {
    var _b = _a === void 0 ? {} : _a, _c = _b.separator, separator = _c === void 0 ? "_" : _c;
    if (path === void 0) { path = []; }
    if (typeof obj !== "object") {
        throw new Error("Can't flatten an object, expected object, but got\" ".concat(typeof obj));
    }
    if (obj === null) {
        throw new Error("Can't flatten null value");
    }
    if (Array.isArray(obj)) {
        throw new Error("Can't flatten array");
    }
    var res = {};
    var _loop_1 = function (key, value) {
        if (typeof value === "object") {
            Object.entries(flatten(value, { separator: separator }, __spreadArray(__spreadArray([], path, true), [key], false))).forEach(function (_a) {
                var subKey = _a[0], subValue = _a[1];
                return (res[key + separator + subKey] = subValue);
            });
        }
        else if (typeof value == "function") {
            throw new Error("Can't flatten object with function as a value of ".concat(key, ". Path to node: ").concat(path.join(".")));
        }
        else {
            res[key] = value;
        }
    };
    for (var _i = 0, _d = Object.entries(obj); _i < _d.length; _i++) {
        var _e = _d[_i], key = _e[0], value = _e[1];
        _loop_1(key, value);
    }
    return res;
}

exports.canonicalSqlTypeHint = canonicalSqlTypeHint;
exports.flatten = flatten;
exports.getSegmentType = getSegmentType;
exports.jitsuToSegment = jitsuToSegment;
exports.removeEmptyFields = removeEmptyFields;
exports.segmentEventsTypes = segmentEventsTypes;
exports.segmentToTable = segmentToTable;
